
#include <stdio.h>

#include "programram.hh"



using namespace vm68k ;




ProgramRAM :: ProgramRAM ( uint32_type startAddr , uint32_type memorySize ,
                    const char * filename , bool debug ) 
                throw ( memory_exception , file_reading_exception )
                    : RAM ( startAddr , memorySize , debug )
{
    FILE * ihex = fopen ( filename , "rt" ) ;

    if ( ihex == NULL )
    {
        throw file_reading_exception () ;
    }

    startPC = 0 ;
    lowestAddrUsed = highestAddrUsed = 0 ;

    while ( ! feof ( ihex ) )
    {
        const uint32_type
                        bytesLimit = 32 ;
        int             bytes [ bytesLimit ] ;
        uint32_type     numberOfBytes , lineAddr , recordType ;
        char            buffer [ 128 ] ;

        fgets ( buffer , 127 , ihex ) ;

        if (( buffer [ 0 ] == ':' )
        && ( sscanf ( buffer , ":%02x%04x%02x" ,
                & numberOfBytes , & lineAddr , & recordType ) == 3 )
        && ( numberOfBytes <= bytesLimit ))
        {
            /* scan data bytes */
            for ( uint32_type i = 0 ; i < numberOfBytes ; i ++ )
            {
                uint32_type startChar = 9 + ( (int) i * 2 ) ;

                if ( ! (( startChar < strlen ( buffer ))
                    && ( sscanf ( & buffer [ startChar ] , "%02x" ,
                            & bytes [ i ] ) == 1 )) )
                {
                    throw file_reading_exception () ;
                }
            }
            if (( recordType == 0 ) // This is a data record
            && ( numberOfBytes > 0 ))
            {
                uint32_type endOfLineAddr = ( lineAddr + numberOfBytes - 1 ) ;

                if ( lineAddr < getLowestAddr ())
                {
                    throw ( address_error ( lineAddr , memory :: WRITE ) ) ;
                } else if ( endOfLineAddr > getHighestAddr ())
                {
                    throw ( address_error ( lineAddr + numberOfBytes - 1 , memory :: WRITE ) ) ;
                }
                    
                for ( uint32_type i = 0 ; i < numberOfBytes ; i ++ )
                {
                    put_8 ( (uint32_type) lineAddr + i , bytes [ i ] ,
                            (function_code) memory :: WRITE ) ;
                }

                if ( endOfLineAddr > highestAddrUsed )
                {
                    highestAddrUsed = endOfLineAddr ;
                } else if ( lineAddr < lowestAddrUsed )
                {
                    lowestAddrUsed = lineAddr ;
                }
            } else if (( recordType == 3 )
            && ( numberOfBytes >= 4 ))
            {
                startPC = bytes [ 3 ] |
                        ( bytes [ 2 ] << 8 ) |
                        ( bytes [ 1 ] << 16 ) |
                        ( bytes [ 0 ] << 24 ) ;
            }
        }
    }
    fclose ( ihex ) ;
}

ProgramRAM :: ~ProgramRAM ()
{
}


