

    carry <= ccr ( 0 ) ;
    overflow <= ccr ( 1 ) ;
    zero <= ccr ( 2 ) ;
    negative <= ccr ( 3 ) ;
    extend <= ccr ( 4 ) ;



    -- Bits 0..6
    alu_seg_1 : entity alu_segment
            generic map ( bits => 7 )
            port map ( input_a => alu_input_a ( 6 downto 0 ) ,
                input_b => alu_input_b ( 6 downto 0 ) ,
                carry_in => alu_carry_in , 
                alu_internal_op => alu_internal_op , 
                carry_out => alu_carry_out_6 ,
                output => alu_int_output ( 6 downto 0 ) ) ;

    -- Bit 7
    alu_seg_2 : entity alu_segment
            generic map ( bits => 1 )
            port map ( input_a => alu_input_a ( 7 downto 7 ) ,
                input_b => alu_input_b ( 7 downto 7 ) ,
                carry_in => alu_carry_out_6 , 
                alu_internal_op => alu_internal_op , 
                carry_out => alu_carry_out_7 ,
                output => alu_int_output ( 7 downto 7 ) ) ;

    -- Bits 8..14
    alu_seg_3 : entity alu_segment
            generic map ( bits => 7 )
            port map ( input_a => alu_input_a ( 14 downto 8 ) ,
                input_b => alu_input_b ( 14 downto 8 ) ,
                carry_in => alu_carry_out_7 , 
                alu_internal_op => alu_internal_op , 
                carry_out => alu_carry_out_14 ,
                output => alu_int_output ( 14 downto 8 ) ) ;

    -- Bit 15
    alu_seg_4 : entity alu_segment
            generic map ( bits => 1 )
            port map ( input_a => alu_input_a ( 15 downto 15 ) ,
                input_b => alu_input_b ( 15 downto 15 ) ,
                carry_in => alu_carry_out_14 , 
                alu_internal_op => alu_internal_op , 
                carry_out => alu_carry_out_15 ,
                output => alu_int_output ( 15 downto 15 ) ) ;

    -- Bits 16..30
    alu_seg_5 : entity alu_segment
            generic map ( bits => 15 )
            port map ( input_a => alu_input_a ( 30 downto 16 ) ,
                input_b => alu_input_b ( 30 downto 16 ) ,
                carry_in => alu_carry_out_15 , 
                alu_internal_op => alu_internal_op , 
                carry_out => alu_carry_out_30 ,
                output => alu_int_output ( 30 downto 16 ) ) ;

    -- Bit 31
    alu_seg_6 : entity alu_segment
            generic map ( bits => 1 )
            port map ( input_a => alu_input_a ( 31 downto 31 ) ,
                input_b => alu_input_b ( 31 downto 31 ) ,
                carry_in => alu_carry_out_30 , 
                alu_internal_op => alu_internal_op , 
                carry_out => alu_carry_out_31 ,
                output => alu_int_output ( 31 downto 31 ) ) ;

    
    alu : process ( alu_mode , extend , alu_internal_op ,
                alu_input_b , alu_int_output ) is
        variable cin : std_logic ;
    begin
        -- Should alu_carry_in be set?
        -- First we look at the extend bit in the CCR. In one
        -- mode this is taken into account.
        if ( alu_mode = ALU_X_FAMILY )
        and ( extend = '1' )
        then
            cin := '1' ;
        else
            cin := '0' ;
        end if ;
       
        -- Then, carry_in is set. The state is inverted if we are
        -- doing a subtraction or comparison, because subtractions
        -- are actually performed as additions with the 2's complement
        -- of input b. Inverting the carry input is necessary to get
        -- the correct 2's complement value.
        if ( alu_internal_op = ALU_INT_CMP )
        or ( alu_internal_op = ALU_INT_SUB )
        or ( alu_internal_op = ALU_INT_REV_SUB )
        or ( alu_internal_op = ALU_INT_REV_CMP )
        then
            alu_carry_in <= not cin ;
        else
            alu_carry_in <= cin ;
        end if ;

        alu_output <= alu_int_output ;
    end process alu ;


    alu_ccr : process ( alu_internal_op , operation_size ,
                alu_carry_out_6 , alu_carry_out_7 ,
                alu_carry_out_14 , alu_carry_out_15 ,
                alu_carry_out_30 , alu_carry_out_31 ,
                alu_int_output , alu_modify_ccrs , ccr ) is
        variable bits_0_to_7_are_zero   : std_logic ;
        variable bits_0_to_15_are_zero  : std_logic ;
        variable bits_0_to_31_are_zero  : std_logic ;
    begin
        -- Set the condition code registers, if the ALU mode
        -- is one in which they may be modified (all internal
        -- operations, PC <- PC + 1 etc, are non-CCR modifying).
        if ( alu_modify_ccrs = '1' )
        then
            if ( alu_int_output ( 7 downto 0 ) = 
                            conv_std_logic_vector ( 0 , 8 ) )
            then
                bits_0_to_7_are_zero := '1' ;
            else
                bits_0_to_7_are_zero := '0' ;
            end if ;

            if ( alu_int_output ( 15 downto 8 ) = 
                            conv_std_logic_vector ( 0 , 8 ) ) 
            and ( bits_0_to_7_are_zero = '1' )
            then
                bits_0_to_15_are_zero := '1' ;
            else
                bits_0_to_15_are_zero := '0' ;
            end if ;

            if ( alu_int_output ( 31 downto 16 ) = 
                            conv_std_logic_vector ( 0 , 16 ) ) 
            and ( bits_0_to_15_are_zero = '1' )
            then
                bits_0_to_31_are_zero := '1' ;
            else
                bits_0_to_31_are_zero := '0' ;
            end if ;

            case operation_size is
            when BYTE =>
                    alu_ccr_output ( 0 ) <= alu_carry_out_7 ;
                    alu_ccr_output ( 1 ) <= alu_carry_out_7 
                                                xor alu_carry_out_6 ;
                    alu_ccr_output ( 2 ) <= bits_0_to_7_are_zero ;
                    alu_ccr_output ( 3 ) <= alu_int_output ( 7 ) ;
            when WORD =>
                    alu_ccr_output ( 0 ) <= alu_carry_out_15 ;
                    alu_ccr_output ( 1 ) <= alu_carry_out_15 
                                                xor alu_carry_out_14 ;
                    alu_ccr_output ( 2 ) <= bits_0_to_15_are_zero ;
                    alu_ccr_output ( 3 ) <= alu_int_output ( 15 ) ;
            when others => -- DWORD =>
                    alu_ccr_output ( 0 ) <= alu_carry_out_31 ;
                    alu_ccr_output ( 1 ) <= alu_carry_out_31 
                                                xor alu_carry_out_30 ;
                    alu_ccr_output ( 2 ) <= bits_0_to_31_are_zero ;
                    alu_ccr_output ( 3 ) <= alu_int_output ( 31 ) ;
            end case ;
        else
            alu_ccr_output ( 3 downto 0 ) <= ccr ( 3 downto 0 ) ;
        end if ;
        alu_ccr_output ( 15 downto 4 ) <= ccr ( 15 downto 4 ) ;
    end process alu_ccr ;

    ccr_update_process : process ( clock , alu_ccr_output ) is
    begin
        if ( clock = '1' )
        and ( clock'event )
        then
            ccr <= alu_ccr_output ;
        end if ;
    end process ccr_update_process ;

    dbcc_monitor : process ( alu_int_output ) is
        constant minus_one : word_register := ( others => '1' ) ;
    begin
        -- This process is solely for the use of DBcc.
        -- If the ALU output is -1 (word), then alu_output_is_minus_one is
        -- set to 1.
        if ( alu_int_output ( 15 downto 0 ) = minus_one )
        then
            alu_output_is_minus_one <= '1' ;
        else
            alu_output_is_minus_one <= '0' ;
        end if ;
    end process dbcc_monitor ;



