-- input.vhd
--
-- $Id: input.vhd,v 1.1 2003/01/17 15:18:39 jdw108 Exp jwhitham $


library ieee ;
use ieee . std_logic_1164 . all ;
use ieee . std_logic_arith . all ;
use ieee . std_logic_unsigned . all ;
use m68k_types . all ;

entity state_machine is
    port ( fast_clock : in std_logic ;
            button : in std_logic ;
            led_display_output : out std_logic_vector ( 6 to 19 ) ;
            right_led_display_output : out std_logic_vector ( 6 to 19 ) ;
            switches : in std_logic_vector ( 6 downto 0 ) ;
            light_1 : out std_logic ;
            light_2 : out std_logic ;
            light_3 : out std_logic ) ;
end entity state_machine ;




architecture basic of state_machine is


-- Include signal definition code

INCLUDE clock.s.vhd
INCLUDE alu.s.vhd
INCLUDE alu_muxes.s.vhd
INCLUDE debugging.s.vhd
INCLUDE do_branch_process.s.vhd
INCLUDE ea_mode_mux_process.s.vhd
INCLUDE memory.s.vhd
INCLUDE operation_size_control_process.s.vhd
INCLUDE register_file.s.vhd
INCLUDE state_machine_controller.s.vhd
INCLUDE defaults.s.vhd
INCLUDE restore_pc_after_immediate_fetch.s.vhd

-- Optimisation code
INSERT OPTIMISATION alu_internal_op
INSERT OPTIMISATION ea_mode
INSERT OPTIMISATION ea_reg

begin


-- Code for CPU components

INCLUDE clock.vhd
INCLUDE alu.vhd
INCLUDE alu_muxes.vhd
INCLUDE debugging.vhd
INCLUDE do_branch_process.vhd
INCLUDE ea_mode_mux_process.vhd
INCLUDE memory.vhd
INCLUDE operation_size_control_process.vhd
INCLUDE register_file.vhd
INCLUDE restore_pc_after_immediate_fetch.vhd




state_machine_process : process ( 

INCLUDE state_machine_sensitivity_list.vhd

state , clock ) is

begin

INCLUDE defaults.vhd

INSERT STATE MACHINE

end process state_machine_process ;

instruction_decoder : process ( instruction_register ) is
    variable ir : word_register ;
begin
    ir := instruction_register ;
    INSERT INSTRUCTION DECODER
end process instruction_decoder ;


INCLUDE state_machine_controller.vhd

end architecture basic ;


