
#include <string.h>


#include "basic_optimisation.h"
#include "utils.h"
#include "definitions.h"



/****** Basic_Optimisation public methods ******/

/** Basic_Optimisation destructor
 *
 * Frees all the strings in the Assertion_Set, if any.
 * The copy of the optimisation name is also deleted.
 */
Basic_Optimisation :: ~Basic_Optimisation ()
{
    Assertion_Set_Iter     iter ;
    
    iter = assertion_set . begin () ;


    while ( iter != assertion_set . end () )
    {
        const char *    item = (* iter) ;

        delete item ;

        iter ++ ;
    }
    delete optimisation_name ;
    delete optimisation_type ;
}


/** Generate_VHDL
 *XXX document
 */
void Basic_Optimisation :: Generate_VHDL ( FILE * output )
{
    fprintf ( output , "\n"
        "subtype param_%s is %s ;\n"
        "function apply_%s ( i : in param_%s ) return param_%s is\n"
        "begin\n" , optimisation_name , optimisation_type ,
        optimisation_name , optimisation_name , optimisation_name ) ;

    if ( assertion_set . empty () )
    {
        fprintf ( output , "-- Optimisation %s is not available.\n"
                    "  return i ;\n" , optimisation_name ) ;
    } else {
        Assertion_Set_Iter      iter ;
        
        /* Generate an if statement of the form:
         * if i = <allowed values>
         * then return i else return <an arbitrary allowed value> end if 
         */

        fprintf ( output , "  if " ) ;
       
        iter = assertion_set . begin () ;

        while ( iter != assertion_set . end () )
        {
            const char *    item = (* iter) ;

            if ( iter != assertion_set . begin () )
            {
                fprintf ( output , " or " ) ;
            }
            fprintf ( output , "( i = %s )" , item ) ;

            iter ++ ;
        }

        const char *    first_item = (* assertion_set . begin () ) ;
        
        fprintf ( output , "\n  then\n"
                           "    return i ;\n"
                           "  else\n"
                           "    return %s ;\n"
                           "  end if ;\n" , first_item ) ;
    }
    fprintf ( output , "end function apply_%s ;\n\n" , optimisation_name ) ;
}


/****** Basic_Optimisation protected methods ******/

/** Add_Assert
 *
 * Allow the specified register, reg_name, to take the given value.
 * The width of the value must be specified.
 */
void Basic_Optimisation :: Add_Assert ( int value , int width )
{
    const char *    bin_str = Binary_To_String ( value , width ) ;
    
    char *          bin_str_with_quotes = 
                        new char [ strlen ( bin_str ) + 3 ] ;

    strcpy ( bin_str_with_quotes , "\"" ) ;
    strcat ( bin_str_with_quotes , bin_str ) ;
    strcat ( bin_str_with_quotes , "\"" ) ;

    Add_Assert ( bin_str_with_quotes ) ;

    delete [] bin_str_with_quotes ; 
    delete [] bin_str ; 
}


/** Add_Assert
 *
 * Allow the specified register, reg_name, to take the given value.
 */
void Basic_Optimisation :: Add_Assert ( const char * value )
{
    /* Attempt to insert value_copy into the assertion_set */
    const char *                    value_copy = Copy_String ( value ) ;
    pair<Assertion_Set_Iter, bool>  insert_result ;

    insert_result = assertion_set . insert ( value_copy ) ;

    MESSAGE3 ( "For opt %s, assertion %s = %s " , 
                    optimisation_name , optimisation_name , value_copy ) ;

    if ( ! insert_result . second )
    {
        /* Failed. Must already be present. */
        delete [] value_copy ;
        MESSAGE3 ( "[already present]\n" ) ;
    } else {
        MESSAGE3 ( "[ok]\n" ) ;
    }
}





