#ifndef CONTROL_H
#define CONTROL_H

#include <stdio.h>
#include <regex.h>

#include "state_machine.h"
#include "opcode_database.h"
#include "state_machine_loader.h"
#include "optimisation.h"

class Control 
{
public:
    Control ( const char * opcode_map_file ,
                    const char * state_machine_directory ,
                    const char * root_vhdl_input_file ,
                    const char * vhdl_input_directory ,
                    const char * initial_state_machine ,
                    const char * required_opcode_list_file ) ;
    virtual ~Control () ;
    void Generate_VHDL ( FILE * output ) ;

private:
    void Add_VHDL_Source_File ( const char * filename , FILE * output ) ;
    void Require_Opcodes_In_File ( const char * file ) ;

    Opcode_Database opcode_database ;
    State_Machine * master_sm ;
    State_Machine_Loader    
                    sm_loader ;
    Optimisation_Manager
                    optimisation_manager ;
    regex_t         include_regex ; 
    regex_t         insert_subtypes_regex ; 
    regex_t         insert_state_machine_regex ; 
    regex_t         insert_instruction_decoder_regex ;
    regex_t         insert_optimisation_regex ;
    regex_t         require_opcode_regex ;
    const char *    root_vhdl_input_file ;
    const char *    vhdl_input_directory ;

    static const char * require_opcode_expression ; 
} ;

#endif




