#ifndef OPTIMISATION_H
#define OPTIMISATION_H

#include <stdio.h>
#include <set>

#include "basic_optimisation.h"
#include "ea_optimisation.h"
#include "ea_reg_optimisation.h"
#include "alu_optimisation.h"
#include "opcode_map_reader.h"


enum Optimisation_Type { EA = 0 , EAREG , 
                ALUOP , NUMBER_OF_OPTIMISATIONS } ;


struct Optimisation_Type_Data
{
    Optimisation_Type   type ;
    const char          name [ 16 ] ;
    const char          codes [ 16 ] ;
} ;


class Optimisation_Record
{
public:
    Optimisation_Record ( const char * name ) ;
    Optimisation_Record ( char c ) ;


    Optimisation_Type Get_Optimisation_Type () const
                { return type ; } ;
    char Get_Subtype () const
                { return subtype ; } ;

    void Print_Info () ;

private:
    Optimisation_Type   type ;
    char                subtype ;

    static const Optimisation_Type_Data opt_data [ NUMBER_OF_OPTIMISATIONS ] ;
} ;




struct Optimisation_Record_Compare
{
    bool operator () ( const Optimisation_Record * r1 , 
                        const Optimisation_Record * r2 ) const
    {
        if ( r1 -> Get_Optimisation_Type () ==  
                r2 -> Get_Optimisation_Type () )
        {
            return ( r1 -> Get_Subtype () < r2 -> Get_Subtype () ) ;
        } else {
            return ( r1 -> Get_Optimisation_Type () ) <
                    ( r2 -> Get_Optimisation_Type () ) ;
        }
    }
} ;



typedef set<Optimisation_Record *, Optimisation_Record_Compare> 
                                            Optimisation_Record_Set ;
typedef Optimisation_Record_Set::iterator Optimisation_Record_Set_Iter ;

class Optimisation_NDFA_Accept_State : public Accept_State
{
public:
    virtual ~Optimisation_NDFA_Accept_State () ;

    Optimisation_Record_Set & Get_Optimisation_Record_Set ()
            { return opts ; } ;
    void Add_Optimisation ( Optimisation_Record * r )
            { opts . insert ( r ) ; } ;

private: 
    Optimisation_Record_Set opts ;
} ;



class Optimisation_Manager : public Opcode_Map_Reader
{
public:
    Optimisation_Manager () ;
    virtual ~Optimisation_Manager () ;

    void Notify ( unsigned opcode ) ;

    void Generate_VHDL ( FILE * output , Optimisation_Record o_type ) ;

    virtual Accept_State * New_Accept_State ( 
                const char * cmp_bit_pattern , 
                const char * opcode_name , 
                const char * dea , 
                const char * micro_sub_name , 
                const char * optimisation_info , 
                const char * comment ) ;


private:
    Basic_Optimisation * optimisation_types [ NUMBER_OF_OPTIMISATIONS ] ;
} ;



#endif


