#ifndef STATE_H
#define STATE_H

#include <stdio.h>


#include <set>
#include <map>
#include <list>
#include <string>

#include "optimisation.h"

/* The label set is used for passing around lists of dependencies on
 * particular state labels */
typedef set<const char *> Label_Set ;
typedef Label_Set :: iterator Label_Set_Iterator ;

class State ;


/* The Command class maintains details of a single command. There are
 * 1 or more commands in a state. */

class Command
{
public:
    enum Command_Type { UNKNOWN_TYPE , CALL , RETURN , 
                JUMP , VHDL , IDECODE } ;


    Command ( Command_Type t , const char * cmd_data = 0L ) ;
    virtual ~Command () ;


    Command_Type Get_Type ()
            { return type ; } ;
    const char * Get_Data ()
            { return data ; } ;

private:
    Command_Type    type ;
    const char *    data ;
} ;

typedef list<Command *> Command_List ;
typedef Command_List :: iterator Command_List_Iterator ;

/* The state map maintains a link between a label name and the state
 * it refers to. */

struct Strings_Less
{
    bool operator () ( string s1 , string s2 ) const
    {
        return strcmp ( s1 . c_str () , s2 . c_str () ) < 0 ;
    }
} ;

typedef map<string , State * , Strings_Less> State_Map ;
typedef State_Map :: iterator State_Map_Iterator ;


/* The state command maintains details of a single state in the state
 * machine. There are 1 or more states in a state machine. */

class State
{
public:
    State () { state_name = 0L ; } ;
    virtual ~State () ;


    void Add_Command ( Command * cmd ) ;
    Label_Set Depends_On () ;
    void Generate_VHDL ( FILE * output , State_Map * definitions ) ;
    void Generate_Link_To_State_VHDL ( FILE * output , const char * indent ) ;

    void Set_Abs_State_Number ( int n )
            { abs_state_number = n ; } ;
    void Set_Width_Of_State_Number ( int n )
            { width_of_state_number = n ; } ;
    void Set_State_Name ( const char * name ) ;
    bool Is_Empty ()
            { return commands . empty () ; } ;


private:
    Command_List    commands ;
    int             abs_state_number ;
    int             width_of_state_number ;
    const char *    state_name ;
} ;

typedef list<State *> State_List ;
typedef State_List :: iterator State_List_Iterator ;



#endif

